<?php
/* 
Plugin Name: LoveClaw
Plugin URI: http://loveclaw.com
Description: Adds custom social buttons at the top of each post/page.
Author: loveclaw.com
Version: 2.22
Author URI: http://loveclaw.com/
*/
include_once(ABSPATH . 'wp-admin/includes/plugin.php');

class viral_buttons
{
	var $loveclaw_words; // emotion button labels
	var $loveclaw_posts; // excluded posts (comma-separated list of IDs)
	var	$loveclaw_pages; // excluded pages
	var	$loveclaw_tags; // excluded pages
	var $loveclaw_categories; // excluded categories
	var $show_auth_error; // can't use file_get_contents
  	
	function viral_buttons()
	{
		$this->loveclaw_words = explode(',',get_option('loveclaw_words'));
		$this->loveclaw_posts = explode(',',get_option('loveclaw_posts'));
		$this->loveclaw_pages = explode(',',get_option('loveclaw_pages'));
		$this->loveclaw_tags = explode(',',get_option('loveclaw_tags'));
		$this->loveclaw_categories = explode(',',get_option('loveclaw_categories'));
   	add_filter('the_content', Array(&$this, 'insert_loveclaw_buttons'));
	}	

	function add_admin_menu()
	{
		$icon_path = get_option('siteurl').'/wp-content/plugins/'.basename(dirname(__FILE__)).'/icons';
		add_options_page('LoveClaw', 'LoveClaw', 10, __FILE__, array($this, 'admin_page'),$icon_path.'/icon.png');
	}
		
  // Check if post/page/etc is excluded
	function is_excluded($post)
	{
	  if (!is_single() && !is_page()) return true;

    $exclusions = is_page() ? $this->loveclaw_pages : $this->loveclaw_posts;
		foreach ($exclusions as $exclusion)
		{
			if ($exclusion == $post->ID)
			  return true;
		}
		if (has_tag($this->loveclaw_tags,$post)) return true;
		if (in_category($this->loveclaw_categories,$post)) return true;			
		
		return false;
	}

  function getDomain()
  {
		$domain = str_replace("http://","",site_url());
		$domain = str_replace("https://","",$domain);
		return $domain;
  }
  
	function generate_buttons()
	{
		global $post;
		if ($this->is_excluded($post)) return;
		return '<div id="loveclaw_wrapper"></div>';
	}
	
	function remove_style_ver( $src )
	{
    $parts = explode( '?', $src );
    return $parts[0];
  }

	function init() 
	{          
	   wp_enqueue_style( 'loveclaw', WP_PLUGIN_URL.'/'.basename(dirname(__FILE__)).'/style.css', NULL );
  }

  // filters the_content. Adds LoveClaw buttons automatically if user has chosen this option.
	function insert_loveclaw_buttons($content)
  {
    $option = get_option('loveclaw_auto_add_buttons');
    $retval = $option == '1' ? $this->generate_buttons().$content : ($option == '2' ? $content.$this->generate_buttons() : $content);
    $retval .= $this->get_js();
    return $retval;
  }
  
  function lc_version()
  {
    $api_version = "wp";
    if (function_exists(get_plugin_data)) {
      $plugin_data = get_plugin_data(__FILE__);
      $api_version .= $plugin_data['Version'];
    }
    return $api_version;
  }
  
  // adds LoveClaw javascript.
	function get_js()
  {
		global $post;
 		$domain = str_replace("http://","",site_url());
		$domain = str_replace("https://","",$domain);
    $retval = "<link rel='stylesheet' href='http://cdn.loveclaw.com/style.css' type='text/css' media='all' />";
    $retval .= "<script type='text/javascript' src='http://cdn.loveclaw.com/buttons.js'></script>";
    $retval .= "<script type='text/javascript'>loveClawOptions.DomainName='".$domain."';";
    $retval .= "loveClawOptions.LicenseKey='".get_option('loveclaw_license_key')."';";
    $retval .= "loveClawOptions.ButtonStyle=".get_option('loveclaw_button_class').";";
    $retval .= "loveClawOptions.HeaderLabel='".get_option('loveclaw_pretext')."';";
    $retval .= "loveClawOptions.SocialSite=".get_option('loveclaw_social_site').";";
    $retval .= 'loveClawOptions.ExitHTML=\''.get_option('loveclaw_exit_html').'\';';
		if (preg_match('/<img (.*?)src=[\'"](https?:\/\/(www\.)?'.str_replace("/","\\/",$domain).'\/(.*?))[\'"]/i',$post->post_content,$matches)) {
      $retval .= "loveClawOptions.ImageURL='".$matches[2]."';";
    }
    $retval .= "loveClawOptions.API='".$this->lc_version()."';";
    $retval .= "loveClawOptions.ButtonLabels=['";
    $retval .= str_replace(",","','", get_option('loveclaw_words'));
    $retval .= "'];</script>";
    return $retval;
  }

	function activate()
	{
		add_option('loveclaw_words', 'I love it,I hate it,Interesting,Shocking,I don\\\'t care');
		add_option('loveclaw_posts', '');
		add_option('loveclaw_pages', '');
		add_option('loveclaw_tags', '');
		add_option('loveclaw_pretext', 'Facebook It:');
		add_option('loveclaw_categories', '');
		add_option('loveclaw_auto_add_buttons', '0');
		add_option('loveclaw_button_class', '2');
		add_option('loveclaw_social_site', '1');
		add_option('loveclaw_license_key', '');
		add_option('loveclaw_image_domain', '');
		add_option('loveclaw_license_status', '0');
		add_option('loveclaw_exit_html', '<div class="thanks">Thanks for Sharing!</div>');
  }

	function deactivate()
	{
	  $key = get_option('loveclaw_license_key');
		file_get_contents("http://loveclaw.com/manageKeys.php?licenseAction=deactivate&licenseKey=".urlencode($key)."&domain=" . urlencode(str_replace("http://","",str_replace("https://","",site_url()))));
		delete_option('loveclaw_words');
		delete_option('loveclaw_posts');
		delete_option('loveclaw_pages');
		delete_option('loveclaw_tags');
		delete_option('loveclaw_pretext');
		delete_option('loveclaw_categories');
		delete_option('loveclaw_auto_add_buttons');
		delete_option('loveclaw_image_domain');
		delete_option('loveclaw_button_class');
		delete_option('loveclaw_social_site');
		delete_option('loveclaw_license_key');
		delete_option('loveclaw_license_status');
		delete_option('loveclaw_exit_html');
	}
	
	function admin_page()
	{
		?>
		<div class='wrap'>
			<h2>
				<?php echo $this->page_title;?>
			</h2>
			<p>
				<?php echo $this->short_description;?>
			</p>		
			
			<?php			
			if ($_POST) {					
   		  if (isset($_POST['loveclaw_key'])) {
			    $key = trim($_POST['loveclaw_key']);
  				update_option('loveclaw_license_key',$key);
  				$url = "http://loveclaw.com/manageKeys.php?licenseAction=activate&api=".$this->lc_version()."&licenseKey=".urlencode($key)."&domain=".urlencode(str_replace("http://","",str_replace("https://","",site_url())));
  				$authStatus = file_get_contents($url);
  				update_option('loveclaw_license_status',$authStatus == "" ? "-1" : $authStatus);  // -1 = unable to call out, ask for alternate method
  				if ($authStatus == 2) { // Twitter only
  				  update_option('loveclaw_social_site', "2");
  			  }
			  }
				if (isset($_POST['loveclaw_words'])) 		update_option('loveclaw_words',$_POST['loveclaw_words']);
				if (isset($_POST['loveclaw_posts'])) 		update_option('loveclaw_posts',$_POST['loveclaw_posts']);
				if (isset($_POST['loveclaw_pages'])) 		update_option('loveclaw_pages',$_POST['loveclaw_pages']);
				if (isset($_POST['loveclaw_tags'])) 			update_option('loveclaw_tags',$_POST['loveclaw_tags']);
				if (isset($_POST['loveclaw_pretext'])) 			update_option('loveclaw_pretext',$_POST['loveclaw_pretext']);
				if (isset($_POST['loveclaw_image_domain'])) 			update_option('loveclaw_image_domain',$_POST['loveclaw_image_domain']);
				if (isset($_POST['loveclaw_categories'])) 	update_option('loveclaw_categories',$_POST['loveclaw_categories']);
				if (isset($_POST['loveclaw_button_class'])) 	update_option('loveclaw_button_class',$_POST['loveclaw_button_class']);
				if (isset($_POST['loveclaw_social_site'])) 	update_option('loveclaw_social_site',$_POST['loveclaw_social_site']);
				if (isset($_POST['loveclaw_auto_add_buttons'])) 	update_option('loveclaw_auto_add_buttons',$_POST['loveclaw_auto_add_buttons']);
				if (isset($_POST['loveclaw_exit_html'])) {
				  $html = str_replace("</","<\/",$_POST['loveclaw_exit_html']);
				  $html = preg_replace("/\r\n|\n\r|\r|\n/"," ",$html);
				  update_option('loveclaw_exit_html',$html);
				}
				?>
				<div class='updated'>Configuration successfully saved.</div>
				<?php
			} 
			$this->view_options_page();
		echo '</div>';
	}

	function view_options_page()
	{
	  $key = get_option('loveclaw_license_key');
	  $keyStatus = get_option('loveclaw_license_status');
	  if ($keyStatus == "0") {
	  ?>
      <h2>LoveClaw Options</h2>
	  	<form id="loveclaw_license_key" method="post" action="">
    		<fieldset class="loveclaw_fieldset">
        	<legend class="loveclaw_legend">License Key</legend>
        	<p><p>You're almost there! You just need to activate your LoveClaw plugin by generating a license key <a href='http://www.loveclaw.com/manageKeys.php'>here</a>.</p><p>Your login details were sent to your Payment/PayPal email address.</p>
          <table class="form-table">
            <tr valign="top">
              <td><input type="text" style="width: 250px;" name="loveclaw_key" value="<?php if ($key) echo $key;?>"/></td>
            </tr>
          </table>
        </fieldset>
     		<div style="position:relative;">
					<input type="submit" value="Activate Key" name="activate-key" class="btn-submit"/>
				</div>
      </form>
   	<?php
   		if ($key != "") { // they've provided a key, but it's invalid
   		  echo "<p>Invalid license key. Log in to your LoveClaw account and <a href='http://www.loveclaw.com/manageKeys.php'>generate a new key here</a>.</p>";
   		}
      return;
	  }
		?>
    <h2>LoveClaw Options</h2>
		<div id="options_form" style="margin:20px;">			
			<form id="loveclaw_options" method="post" action="">
    		<fieldset class="loveclaw_fieldset">
        	<legend class="loveclaw_legend">License Key</legend>
          <table class="form-table">
            <tr valign="top"><td width='150'>License Key:</td><td><?php echo get_option('loveclaw_license_key'); ?></td></tr>
            <tr valign="top"><td>License Status:</td>
            	<?php if ($keyStatus == "-1") {
            	  update_option('loveclaw_license_status', "3"); // show the error once. then assume all sites valid
        	  		$domain = $this->getDomain();
		  					echo "<td><p style='color:#990000'>Unable to validate license key. Please <a target='_blank' href='http://loveclaw.com/members/reports/activateKey.php?licenseKey=".$key."&domain=".$domain."'>click here</a> to attempt validation on the server. If this does not work, please contact <a href='mailto:support@loveclaw.com'>support@loveclaw.com</a> and ask them to activate key '".$key."' for domain '".$domain."'.</p></td>";
            	}
            	else { ?>
                <td>Activated</td>
                <?php if ($keyStatus != "3") { // if recent upgrade from one social site to both, key status must be refreshed ?>
                <td>
                  <input type="hidden" name="loveclaw_key" value="<?php if ($key) echo $key;?>"/>
      					  <input type="submit" value="Re-check" name="reauth-submit" class="btn-submit"/>
      					</td>
            	<?php } } ?>
            	</td></tr>
          </table>
        </fieldset>
    		<fieldset class="loveclaw_fieldset">
        	<legend class="loveclaw_legend">Emotion Button Labels</legend>
        	<p>Enter a comma-separated list of labels. A button will be created for each label.</p>
          <table class="form-table">
            <tr valign="top">
              <td><input type="text" style="width: 450px;" name="loveclaw_words" value="<?php if (get_option('loveclaw_words')) echo stripslashes(get_option('loveclaw_words'));?>" class="words"/></td>
            </tr>
          </table>
        </fieldset>
    		<fieldset class="loveclaw_fieldset">
        	<legend class="loveclaw_legend">Preamble</legend>
        	<p>Enter preamble text. This text will be displayed to the left of the buttons.</p>
          <table class="form-table">
            <tr valign="top">
              <td><input type="text" style="width: 450px;" name="loveclaw_pretext" value="<?php if (get_option('loveclaw_pretext')) echo stripslashes(get_option('loveclaw_pretext'));?>" class="words"/></td>
            </tr>
          </table>
        </fieldset>
    		<fieldset class="loveclaw_fieldset">
        	<legend class="loveclaw_legend">Image Domain</legend>
        	<p>The domain where images are hosted, if different from the blog's domain (for example, if images are hosted on a CDN). Enter without 'http://' prefix.</p>
          <table class="form-table">
            <tr valign="top">
              <td><input type="text" style="width: 450px;" name="loveclaw_image_domain" value="<?php if (get_option('loveclaw_image_domain')) echo get_option('loveclaw_image_domain');?>" class="words"/></td>
            </tr>
          </table>
        </fieldset>
    		<fieldset class="loveclaw_fieldset">
        	<legend class="loveclaw_legend">Exclusions</legend>
        	<p>Exclude specific posts, pages or tags from having LoveClaw buttons. Enter a comma-separated list of post/page/tag IDs to exclude.</p>
          <table style="width:100%" class="form-table">
    				<div class="checkboxes">
            <tr valign="top">
              <th style="width:40%" scope="row">Hide buttons on these posts:</th>
              <td><input type="text" style="width: 450px;" name="loveclaw_posts" value="<?php if (get_option('loveclaw_posts')) echo get_option('loveclaw_posts');?>"/></td>
            </tr>
            <tr valign="top">
              <th scope="row">Hide buttons on these pages:</th>
              <td><input type="text" style="width: 450px;" name="loveclaw_pages" value="<?php if (get_option('loveclaw_pages')) echo get_option('loveclaw_pages');?>"/></td>
            </tr>
            <tr valign="top">
              <th scope="row">Hide buttons on posts with these tags:</th>
              <td><input type="text" style="width: 450px;" name="loveclaw_tags" value="<?php if (get_option('loveclaw_tags')) echo get_option('loveclaw_tags');?>"/></td>
            </tr>
            <tr valign="top">
              <th scope="row">Hide buttons on posts in these categories:</th>
              <td><input type="text" style="width: 450px;" name="loveclaw_categories" value="<?php if (get_option('loveclaw_categories')) echo get_option('loveclaw_categories');?>"/></td>
            </tr>
          </table>
        </fieldset>
        <fieldset class="loveclaw_fieldset">
        	<legend class="loveclaw_legend">Button Style</legend>
        	<p>Choose one of LoveClaw's pre-defined button styles here.<br/></p>
          <table style="width:100%" class="form-table">
            <?php
           	$numButtons = ($keyStatus == "3") ? 30 : 20;
           	$row = 0;
            while(true) { ?>
            <tr valign="top">
              <?php
              for($col = 1; $col <= 3; $col++) {
                $buttonNum = $row * 3 + $col;
                if ($buttonNum > $numButtons)
                  break;
                 ?>
              <td width="20"><input name="loveclaw_button_class" type="radio" value="<?php echo $buttonNum; ?>" <?php checked($buttonNum, get_option( 'loveclaw_button_class' ) ); ?> /></td>
              <td scope="row">
                <a class="loveclaw_button loveclaw_button<?php echo $buttonNum; ?>">
                  <span class="prebutton"></span><span class="button_left"><span class="button_right"><span class="button_center">I Love It!</span></span></span>
                </a>
              </td>
          <?php }
            echo "</tr>";
            $row++;
            if ($buttonNum > $numButtons)
              break;
          } ?>
          </table>
        </fieldset>
        <fieldset class="loveclaw_fieldset">
        	<legend class="loveclaw_legend">Auto-Insert</legend>
        	<p>LoveClaw can automatically insert buttons into your post content. Though this is the simplest way to add these buttons to your site, LoveClaw's choice of button placement may not be right for your site.</p><p>Alternatively, you can control exactly where the buttons will be placed by adding a line of PHP code to your theme manually.</p>
          <table style="width:100%" class="form-table">
            <tr valign="top">
              <td width="20"><input name="loveclaw_auto_add_buttons" type="radio" value="1" <?php checked( '1', get_option( 'loveclaw_auto_add_buttons' ) ); ?> /></td>
              <td scope="row">Let LoveClaw automatically insert buttons at the TOP of all posts</td>
            </tr>
            <tr valign="top">
              <td width="20"><input name="loveclaw_auto_add_buttons" type="radio" value="2" <?php checked( '2', get_option( 'loveclaw_auto_add_buttons' ) ); ?> /></td>
              <td scope="row">Let LoveClaw automatically insert buttons at the BOTTOM of all posts</td>
            </tr>
            <tr valign="top">
              <td><input name="loveclaw_auto_add_buttons" type="radio" value="0" <?php checked( '0', get_option( 'loveclaw_auto_add_buttons' ) ); ?> /></td>
              <td scope="row">I will add the following PHP code to my theme in order to display LoveClaw Buttons:<br/>&nbsp;&nbsp;<em>&lt;?php if (isset($loveclaw)) echo $loveclaw->generate_buttons(); ?&gt;</em></td>
            </tr>
          </table>
        </fieldset>
       	<?php if ($keyStatus == "3") { // both sites licensed ?>
        <fieldset class="loveclaw_fieldset">
        	<legend class="loveclaw_legend">Social Media</legend>
        	<p>LoveClaw can automatically direct the user to a social site of your choice. Or, you can prompt the user to choose a social site after clicking on a LoveClaw button.</p><br/>
        	<p>After a user clicks on a LoveClaw Button:</p>
          <table style="width:100%" class="form-table">
            <tr valign="top">
              <td width="20"><input name="loveclaw_social_site" type="radio" value="1" <?php checked( '1', get_option( 'loveclaw_social_site' ) ); ?> /></td>
              <td scope="row">Direct them to Facebook</td>
            </tr>
            <tr valign="top">
              <td width="20"><input name="loveclaw_social_site" type="radio" value="2" <?php checked( '2', get_option( 'loveclaw_social_site' ) ); ?> /></td>
              <td scope="row">Direct them to Twitter</td>
            </tr>
            <tr valign="top">
              <td><input name="loveclaw_social_site" type="radio" value="3" <?php checked( '3', get_option( 'loveclaw_social_site' ) ); ?> /></td>
              <td scope="row">Allow them to choose a social media site</td>
            </tr>
          </table>
          <br/><br/>
        	<p>If you've given the user a choice in the setting above, LoveClaw will show a thank-you message after the user has shared your page. You can customize this message by providing HTML below:</p><br/>
          <textarea type="text" rows="4" cols="80" name="loveclaw_exit_html"><?php if (get_option('loveclaw_exit_html')) echo stripslashes(get_option('loveclaw_exit_html'));?></textarea>
        </fieldset>
        <?php } ?>
     		<div style="position:relative;">
					<input type="submit" value="Save" name="options-submit" class="btn-submit"/>
				</div>
			</form>													
		</div>	
		
		<?php
	}
}

$loveclaw = new viral_buttons();
$path_to_php_file_plugin = dirname(__FILE__);
add_action('admin_menu', array($loveclaw, 'add_admin_menu'));
add_action('deactivate_' . $path_to_php_file_plugin, array($loveclaw, 'deactivate'));
add_action('activate_' . $path_to_php_file_plugin, array($loveclaw, 'activate'));
register_activation_hook( __FILE__, array($loveclaw, 'activate'));
register_deactivation_hook(__FILE__, array($loveclaw, 'deactivate'));
add_action('admin_init', array($loveclaw,'init'));
add_action('admin_print_styles', array($loveclaw,'init')); // hook for admin css file
?>